package com.broadlink.flutter.blsdk_plugin;

import android.content.Context;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/** BlsdkPlugin */
public class BlsdkPlugin implements FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;

  private final String LOGTAG=BlsdkPlugin.class.getSimpleName();
  private Context applicationContext;
  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "blsdk_plugin");
    channel.setMethodCallHandler(this);
    applicationContext=flutterPluginBinding.getApplicationContext();
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    }
    else if (call.method.equals(NativeInterface.INIT_BLSDK)) {
      String initData=call.argument("data");
      BLLogUtils.d(LOGTAG,"onMethodCall current thread name:"+Thread.currentThread().getName());
      boolean initResult=BroadlinkSDKTools.getInstance(applicationContext).initBLSDK(initData);
      result.success(initResult);
    }
    else if (call.method.equals(NativeInterface.CONTROL_COMMAND)) {
      String data=call.argument("data");
      String commandResult=BroadlinkSDKTools.getInstance(applicationContext).controlCommand(data);
      result.success(commandResult);
    }
    else if (call.method.equals(NativeInterface.QUERY_COMMAND)) {
      String data=call.argument("data");
      String commandResult=BroadlinkSDKTools.getInstance(applicationContext).queryCommand(data);
      result.success(commandResult);
    }
    else if (call.method.equals(NativeInterface.START_PROBE)) {
      int probe_interval=call.argument("probe_interval");
      BroadlinkSDKTools.getInstance(applicationContext).statrProbe(probe_interval);
    }
    else if (call.method.equals(NativeInterface.STOP_PROBE)) {
      BroadlinkSDKTools.getInstance(applicationContext).stopProbe();
    }
    else if(call.method.equals(NativeInterface.GET_PROBED_DEVICE_CACHE)){
      String probedResult = BroadlinkSDKTools.getInstance(applicationContext).getProbedDeviceCache();
      result.success(probedResult);
    }
    else if(call.method.equals(NativeInterface.CLEAR_PROBED_DEVICE_CACHE)){
      BroadlinkSDKTools.getInstance(applicationContext).clearProbedDeviceCache();
    }
    else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
    applicationContext=null;
  }
}
