package com.broadlink.flutter.blsdk_plugin;

import android.content.Context;

import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import cn.com.broadlink.base.BLConfigParam;
import cn.com.broadlink.sdk.BLLet;
import cn.com.broadlink.sdk.data.controller.BLDNADevice;
import cn.com.broadlink.sdk.interfaces.controller.BLDeviceScanListener;
import cn.com.broadlink.sdk.param.controller.BLStdControlParam;
import cn.com.broadlink.sdk.result.controller.BLStdControlResult;

public class BroadlinkSDKTools {
    private final String LOGTAG= BroadlinkSDKTools.class.getSimpleName();
    private Context mContext;
    private static BroadlinkSDKTools instance;
    private DeviceManager deviceManager;
    private BroadlinkSDKTools(){}

    public BroadlinkSDKTools(Context mContext) {
        this.mContext = mContext;
        deviceManager=new DeviceManager();
    }

    public static BroadlinkSDKTools getInstance(Context context){
        if(instance==null){
            synchronized (BroadlinkSDKTools.class){
                if(instance==null){
                    instance=new BroadlinkSDKTools(context);
                }
            }
        }
        return instance;
    }
    /**
     *initData: {
     * 	"local_timeout": "",
     * 	"remote_timeout": "",
     * 	"device_tcp_server": "",
     * 	"device_http_server": "",
     * 	"lid": "",
     * 	"license": "",
     * 	"channel_id": "",
     * 	"app_host": ""
     * }
     * */
    public boolean initBLSDK(String initData){
        JsonParser jsonParser=new JsonParser();
        try {
            BLConfigParam blConfigParam = new BLConfigParam();
            blConfigParam.put(BLConfigParam.CONTROLLER_LOCAL_TIMEOUT, "3000");
            blConfigParam.put(BLConfigParam.CONTROLLER_REMOTE_TIMEOUT, "5000");
            blConfigParam.put(BLConfigParam.CONTROLLER_LOG_LEVEL, BuildConfig.DEBUG ? "4" : "0");
            blConfigParam.put(BLConfigParam.CONTROLLER_JNI_LOG_LEVEL, BuildConfig.DEBUG ? "4" : "0");

            JsonObject initDataJson=jsonParser.parse(initData).getAsJsonObject();
            //本地和远程超时时间
            JsonElement element=initDataJson.get("local_timeout");
            if(element!=null){
                blConfigParam.put(BLConfigParam.CONTROLLER_LOCAL_TIMEOUT, element.getAsString());
            }
            element=initDataJson.get("remote_timeout");
            if(element!=null){
                blConfigParam.put(BLConfigParam.CONTROLLER_REMOTE_TIMEOUT, element.getAsString());
            }
            //指定设备连接的服务器
            JsonObject deviceConnectDataJson = new JsonObject();
            element=initDataJson.get("device_tcp_server");
            if(element!=null){
                if(deviceConnectDataJson==null) {deviceConnectDataJson = new JsonObject();}
                deviceConnectDataJson.add("tcp", element);
            }
            element=initDataJson.get("device_http_server");
            if(element!=null){
                if(deviceConnectDataJson==null) {deviceConnectDataJson = new JsonObject();}
                deviceConnectDataJson.add("http", element);
            }
            element=initDataJson.get("lid");
            if(element!=null){
                if(deviceConnectDataJson==null) {deviceConnectDataJson = new JsonObject();}
                deviceConnectDataJson.add("companyid", element);
            }
            if(deviceConnectDataJson!=null){
                blConfigParam.put(BLConfigParam.DEVICE_CONNECTION_SERVER_HOST, deviceConnectDataJson.toString());
            }
            //是否指定sdk连接使用app指定服务器
            blConfigParam.put(BLConfigParam.APP_SERVICE_ENABLE, "1");
            //指定脚本下载平台
            blConfigParam.put(BLConfigParam.CONTROLLER_SCRIPT_DOWNLOAD_VERSION, "1");
            //网络模式
            blConfigParam.put(BLConfigParam.CONTROLLER_NETMODE, "3");
            //  /** Probe支持的协议 **/
            blConfigParam.put(BLConfigParam.CONTROLLER_SUPPORT_PROTOCOL, "1");

            //license
            String license=null;
            element=initDataJson.get("license");
            if(element!=null){
                license=element.getAsString();
            }
            //发布渠道，暂时无用
            String channelId=null;
            element=initDataJson.get("channel_id");
            if(element!=null){
                channelId=element.getAsString();
            }
            BLLet.init(mContext, license, channelId, blConfigParam);
            return true;
        }catch (Exception e){
            BLLogUtils.e(LOGTAG,"initBLSDK fail",e);
            JsonObject resultJson=new JsonObject();
            resultJson.addProperty("status", ResultStatus.InitSdkDataError);
            resultJson.addProperty("msg","init BLSDK data error");
        }
        return false;
    }

    public void statrProbe(int probeInterval){
        BLLet.Controller.startProbe(probeInterval);
        BLLet.Controller.setOnDeviceScanListener(new BLDeviceScanListener() {
            @Override
            public void onDeviceUpdate(BLDNADevice device, boolean isNewDevice) {
                deviceManager.addProbedBlDevice(device);
            }
        });
    }

    public void stopProbe(){
        BLLet.Controller.stopProbe();
    }

    public String controlCommand(String command){
        String gateway_id=null;
        String device_id=null;
        String commandStr = null;
        BLConfigParam blConfigParam=null;
        JsonParser jsonParser=new JsonParser();
        try {
            JsonObject commandJson=jsonParser.parse(command).getAsJsonObject();
            JsonElement element=commandJson.get("gateway_id");
            if(element!=null){
                 gateway_id=element.getAsString();
            }
            element=commandJson.get("did");
            if(element!=null){
                device_id=element.getAsString();
            }
            //控制命令
            element=commandJson.get("command");
            if(element!=null){
                JsonObject commandObject  = element.getAsJsonObject();
                Iterator<Map.Entry<String, JsonElement>> iterator = commandObject.entrySet().iterator();
                HashMap<String,Object> commandMap = null;
                while (iterator.hasNext()){
                    if(commandMap==null){
                        commandMap = new HashMap<>();
                    }
                    Map.Entry<String, JsonElement> elementEntry=iterator.next();
                    commandMap.put(elementEntry.getKey(),elementEntry.getValue());
                }
                if(commandMap!=null){
                    BLStdControlParam blStdControlParam = BLDataTranslateTool.setDevStatus(commandMap);
                    commandStr=new Gson().toJson(blStdControlParam);
                }
            }
            //控制参数
            JsonElement control_params_element = commandJson.get("control_params");
            if(control_params_element!=null){
                JsonObject paramObject  = control_params_element.getAsJsonObject();
                Iterator<Map.Entry<String, JsonElement>> iterator = paramObject.entrySet().iterator();
                while (iterator.hasNext()){
                    if(blConfigParam==null){
                        blConfigParam = new BLConfigParam();
                    }
                    Map.Entry<String, JsonElement> elementEntry=iterator.next();
                    blConfigParam.put(elementEntry.getKey(),elementEntry.getValue().getAsString());
                }
            }

            String did=gateway_id==null?device_id:gateway_id;
            String sdid=gateway_id==null?null:device_id;
            //开始发送控制
            BLStdControlResult result=BLLet.Controller.dnaControl(did,sdid,commandStr,blConfigParam);
            if(result!=null){
                JsonObject resultJson=new JsonObject();
                resultJson.addProperty("status",result.getStatus());
                resultJson.addProperty("msg",result.getMsg());
                if(result.getData()!=null){
                    JsonElement dataElement=jsonParser.parse(new Gson().toJson(result.getData()));
                    resultJson.add("data",dataElement);
                }
                return resultJson.toString();
            }
            return null;
        }catch (Exception e){
            //表示命令格式错误
            BLLogUtils.e(LOGTAG,"send Command error",e);
            JsonObject resultJson=new JsonObject();
            resultJson.addProperty("status", ResultStatus.CommandDataError);
            resultJson.addProperty("msg","send Command data error");
            return resultJson.toString();
        }
    }

    public String queryCommand(String command){
        String gateway_id=null;
        String device_id=null;
        String commandStr = null;
        BLConfigParam blConfigParam=null;
        JsonParser jsonParser=new JsonParser();
        try {
            JsonObject commandJson=jsonParser.parse(command).getAsJsonObject();
            JsonElement element=commandJson.get("gateway_id");
            if(element!=null){
                gateway_id=element.getAsString();
            }
            element=commandJson.get("did");
            if(element!=null){
                device_id=element.getAsString();
            }
            //查询命令
            BLStdControlParam blStdControlParam = BLDataTranslateTool.queryDeviceStatus();
            commandStr=new Gson().toJson(blStdControlParam);

            JsonElement control_params_element = commandJson.get("control_params");
            if(control_params_element!=null){
                JsonObject paramObject  = control_params_element.getAsJsonObject();
                Iterator<Map.Entry<String, JsonElement>> iterator = paramObject.entrySet().iterator();
                while (iterator.hasNext()){
                    if(blConfigParam==null){
                        blConfigParam = new BLConfigParam();
                    }
                    Map.Entry<String, JsonElement> elementEntry=iterator.next();
                    blConfigParam.put(elementEntry.getKey(),elementEntry.getValue().getAsString());
                }
            }

            String did=gateway_id==null?device_id:gateway_id;
            String sdid=gateway_id==null?null:device_id;
            //开始查询
            BLStdControlResult result=BLLet.Controller.dnaControl(did,sdid,commandStr,blConfigParam);
            if(result!=null){
                JsonObject resultJson=new JsonObject();
                resultJson.addProperty("status",result.getStatus());
                resultJson.addProperty("msg",result.getMsg());
                if(result.getData()!=null){
                    JsonElement dataElement=jsonParser.parse(new Gson().toJson(result.getData()));
                    resultJson.add("data",dataElement);
                }
                return resultJson.toString();
            }
            return null;
        }catch (Exception e){
            //表示命令格式错误
            BLLogUtils.e(LOGTAG,"send Command error",e);
            JsonObject resultJson=new JsonObject();
            resultJson.addProperty("status", ResultStatus.CommandDataError);
            resultJson.addProperty("msg","query Command data error");
            return resultJson.toString();
        }
    }

    public String getProbedDeviceCache(){
        List<BLDNADevice>  bldnaDeviceList = deviceManager.getProbedBldevice();
        JsonArray deviceArray=new JsonArray();
        for(BLDNADevice bldnaDevice:bldnaDeviceList){
            JsonObject deviceJson=new JsonObject();
            deviceJson.addProperty("gateway_id",bldnaDevice.getpDid());
            deviceJson.addProperty("did",bldnaDevice.getDid());
            deviceJson.addProperty("pid",bldnaDevice.getPid());
            deviceJson.addProperty("name",bldnaDevice.getName());
            deviceJson.addProperty("key",bldnaDevice.getKey());
            deviceArray.add(deviceJson);
        }
        return deviceArray.toString();
    }

    public void clearProbedDeviceCache(){
        deviceManager.clearProbedDevice();
    }
}
